package Search::Xapian::Weight;

=head1 NAME

Search::Xapian::Weight - base class for Weighting schemes.

=head1 DESCRIPTION

This is an asbtract base class for weighting schemes in Xapian.

=head1 SEE ALSO

L<Search::Xapian>,L<Search::Xapian::BoolWeight>,L<Search::Xapian::BM25Weight>

=cut

1;
